
var Glitter = require('glitter')
var debug = require('debug')('normalize-proxy:app:versions')

var route = require('./route')
var utils = require('../lib/utils')
var cacheControl = require('../config').cacheControl.versions

var match = route(route.project + '/versions.json')
var versionsOf = utils.versionsOf
var calculate = utils.shasum

/**
 * Return a list of all currently installed versions of a repository.
 */

module.exports = function* (next) {
  var params = match(this.request.path)
  if (!params) return yield* next

  // pull from source
  var pull = 'pull' in this.request.query

  var res = this.uri.parseRemote(this.request.path)
  var remote = res[0]
  if (!remote.git) {
    var versions = yield* versionsOf(this.uri.local(res.slice(0, 3))) // only gets the latest version
    return respond.call(this, versions)
  }


  var glitter = Glitter(remote.name, res[1], res[2])
  if (yield glitter.isInstalled()) {
    if (pull) yield glitter.update()
  } else {
    try {
      yield glitter.install()
    } catch (err) {
      debug('%o', err.stack)
      return respond.call(this, [])
    }
  }
  var versions = yield glitter.getVersions()
  return respond.call(this, versions)
}

function respond(versions) {
  this.response.body = versions
  this.response.etag = calculate(JSON.stringify(versions))
  this.response.set('Cache-Control', cacheControl)
  if (this.request.fresh) this.response.status = 304
  else if (!versions.length) this.response.status = 404
}
