
var debug = require('debug')('normalize-proxy:resolve:git')
var exec = require('child_process').exec
var assert = require('http-assert')
var Glitter = require('glitter')
var semver = require('semver')
var fs = require('mz/fs')

// fucking circular dependency!
module.exports = _resolve

var strictVersion = require('../utils').strictVersion
var normalize = require('../normalize')
var resolve = require('./resolve')
var uris = require('../uri')

function* _resolve(uri) {
  var remote = uri[0]
  var owner = uri[1]
  var project = uri[2]
  var range = uri[3]
  var tail = uri[4]
  var glitter = Glitter(remote.name, owner, project)
  debug('glitter: %o', glitter)

  // always install the repo locally
  if (!(yield glitter.isInstalled())) {
    try {
      debug('installing %s:%s/%s', remote.name, owner, project)
      yield glitter.install()
      debug('installed %s:%s/%s', remote.name, owner, project)
    } catch (err) {
      // we should check that this is actually a 404 error
      err.status = 404
      throw err
    }
  }

  var reference
  var version
  if (semver.validRange(range)) {
    // lookup by version
    debug('getting versions: %s:%s/%s@%s', remote.name, owner, project, range)
    var versions = yield glitter.getVersions()
    reference = semver.maxSatisfying(versions, range)
    if (!reference) assert(false, 404, 'no satisfying versions found')
    version = strictVersion(reference)
    debug('resolved version: %s:%s/%s@%s -> %s', remote.name, owner, project, range, version)
  } else {
    // lookup by head
    try {
      // i think this will error if no references are found
      debug('getting reference: %s:%s/%s@%s', remote.name, owner, project, range)
      reference = yield glitter.show(range, true)
    } catch (err) {
      err.status = 404
      throw err
    }
    if (!reference) assert(false, 404, 'reference not found')
  }

  // output folder
  var out = uris.local(remote, owner, project, version)
  if (resolve.installing[out]) {
    debug('waiting to be installed: %s', out)
    yield resolve.await(out)
  } else if (!(yield fs.exists(out))) {
    resolve.installing[out] = true
    try {
      debug('copying %s:%s/%s@%s to %s', remote.name, owner, project, reference, out)
      yield glitter.copy(reference, out)
    } catch (err) {
      // delete the folder
      exec('rm -rf ' + out, noop)
      // will emit an error on any listeners
      resolve.emit(out, err)
      throw err
    } finally {
      delete resolve.installing[out]
    }
    debug('downloaded %s', out)
    yield* normalize(out)
    debug('normalized %s', out)
    resolve.emit(out)
    debug('installed %s/%s/&s@%s', remote.hostname, owner, project, version)
  }

  return [remote, owner, project, version || reference, tail]
}

function noop() {}
